@@ -147,6 +147,33 @@ func TestFlagSimple1(t *testing.T) {
 		t.Errorf("Test %s failed with flag2 being true", t.Name())
 		return
 	}
+
+	if args := p.GetArgs(); args == nil {
+		t.Errorf("Test %s failed with args empty", t.Name())
+	} else if len(args) != 3 { // our two and -h
+		t.Errorf("Test %s failed with wrong len", t.Name())
+	} else {
+		got := 0
+		for _, arg := range args {
+			switch arg.GetLname() {
+			case "flag-arg1":
+				if *arg.GetResult().(*bool) != *flag1 {
+					t.Errorf("Test %s failed with wrong arg value", t.Name())
+				}
+				got += 3
+			case "flag-arg2":
+				if *arg.GetResult().(*bool) != *flag2 {
+					t.Errorf("Test %s failed with wrong arg value", t.Name())
+				}
+				got += 5
+			case "help":
+				got += 11
+			}
+		}
+		if got != 19 {
+			t.Errorf("Test %s failed with wrong args found", t.Name())
+		}
+	}
 }
 
 func TestFlagSimple2(t *testing.T) {
@@ -192,6 +219,7 @@ func TestFlagSimple2(t *testing.T) {
 		t.Errorf("Test %s failed with flag3 being false", t.Name())
 		return
 	}
+
 }
 
 func TestLongFlagEqualChar(t *testing.T) {
@@ -697,6 +725,33 @@ func TestStringSimple1(t *testing.T) {
 		t.Errorf("Test %s failed. Want: [%s], got: [%s]", t.Name(), "\"\"", *s1)
 		return
 	}
+
+	if args := p.GetArgs(); args == nil {
+		t.Errorf("Test %s failed with args empty", t.Name())
+	} else if len(args) != 3 { // our two + help
+		t.Errorf("Test %s failed with wrong len", t.Name())
+	} else {
+		got := 0
+		for _, arg := range args {
+			switch arg.GetLname() {
+			case "flag-arg1":
+				if *arg.GetResult().(*string) != *s1 {
+					t.Errorf("Test %s failed with wrong arg value", t.Name())
+				}
+				got += 3
+			case "flag-arg2":
+				if *arg.GetResult().(*string) != "" {
+					t.Errorf("Test %s failed with non-nil result", t.Name())
+				}
+				got += 5
+			case "help":
+				got += 11
+			}
+		}
+		if got != 19 {
+			t.Errorf("Test %s failed with wrong args found", t.Name())
+		}
+	}
 }
 
 func TestStringSimple2(t *testing.T) {
@@ -1931,6 +1986,26 @@ func TestFlagDefaultValuePass(t *testing.T) {
 
 	p := NewParser("progname", "Prog description")
 
+	f := p.Flag("f", "flag", &Options{Default: false})
+
+	err := p.Parse(testArgs)
+
+	// Should fail on failure
+	if err != nil {
+		t.Error(err.Error())
+	}
+
+	// Should fail if not false
+	if *f != false {
+		t.Errorf("expected [false] but found [%t]", *f)
+	}
+}
+
+func TestFlagDefaultValueShouldIgnoreTrue(t *testing.T) {
+	testArgs := []string{"progname"}
+
+	p := NewParser("progname", "Prog description")
+
 	f := p.Flag("f", "flag", &Options{Default: true})
 
 	err := p.Parse(testArgs)
@@ -1940,24 +2015,9 @@ func TestFlagDefaultValuePass(t *testing.T) {
 		t.Error(err.Error())
 	}
 
-	// Should fail if not true
-	if *f != true {
-		t.Errorf("expected [true], got [%t]", *f)
-	}
-}
-
-func TestFlagDefaultValueFail(t *testing.T) {
-	testArgs := []string{"progname"}
-
-	p := NewParser("progname", "Prog description")
-
-	_ = p.Flag("f", "flag", &Options{Default: "string"})
-
-	err := p.Parse(testArgs)
-
-	// Should pass on failure
-	if err == nil || err.Error() != "cannot use default type [string] as value of pointer with type [*bool]" {
-		t.Errorf("Test %s failed: expected error [%s], got error [%+v]", t.Name(), "cannot use default type [string] as value of pointer with type [*bool]", err)
+	// Should fail if not false
+	if *f != false {
+		t.Errorf("expected [false] but found [%t]", *f)
 	}
 }
 
@@ -1987,7 +2047,7 @@ func TestStringDefaultValueFail(t *testing.T) {
 
 	p := NewParser("progname", "Prog description")
 
-	_ = p.String("s", "string", &Options{Default: false})
+	_ = p.String("s", "string", &Options{Default: true})
 
 	err := p.Parse(testArgs)
 
@@ -2691,3 +2751,483 @@ func TestCommandHelpSetSnameOnly(t *testing.T) {
 		t.Error("Help arugment names should have defaulted")
 	}
 }
+
+func TestCommandPositional(t *testing.T) {
+	testArgs1 := []string{"pos", "heyo"}
+	parser := NewParser("pos", "")
+	strval := parser.StringPositional(nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "heyo" {
+		t.Errorf("Strval did not match expected")
+	}
+}
+
+// Lightly test the Options field
+func TestCommandPositionalOptions(t *testing.T) {
+	testArgs1 := []string{"pos", "heyo"}
+	parser := NewParser("pos", "")
+	validated := false
+	strval := parser.StringPositional(&Options{Validate: func(args []string) error { validated = true; return nil }})
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "heyo" {
+		t.Errorf("Strval did not match expected")
+	} else if !validated {
+		t.Errorf("Validate function not run")
+	}
+}
+
+func TestCommandPositionalUnsatisfied(t *testing.T) {
+	errArgs1 := []string{"pos", "--test1"}
+	parser := NewParser("pos", "")
+	strval := parser.StringPositional(nil)
+	flag1 := parser.Flag("", "test1", nil)
+
+	if err := parser.Parse(errArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "" {
+		t.Errorf("Strval nonempty")
+	} else if parser.GetArgs()[0].GetParsed() {
+		t.Errorf("Strval was parsed")
+	} else if *flag1 != true {
+		t.Errorf("flag not set")
+	}
+}
+
+func TestCommandPositionalUnsatisfiedDefault(t *testing.T) {
+	errArgs1 := []string{"pos"}
+	parser := NewParser("pos", "")
+	defval := "defaultation"
+	strval := parser.StringPositional(&Options{Default: defval})
+
+	if err := parser.Parse(errArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != defval {
+		t.Errorf("Strval (%s) != (%s)", *strval, defval)
+	}
+}
+
+func TestCommandPositionals(t *testing.T) {
+	testArgs1 := []string{"posint", "5", "abc", "1.0"}
+	parser := NewParser("posint", "")
+	intval := parser.IntPositional(&Options{Required: false})
+	strval := parser.StringPositional(nil)
+	floatval := parser.FloatPositional(&Options{Default: 1.5})
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *intval != 5 {
+		t.Error("Intval did not match expected")
+	} else if *strval != "abc" {
+		t.Error("Strval did not match expected")
+	} else if *floatval != 1.0 {
+		t.Error("Floatval did not match expected")
+	}
+}
+
+func TestCommandPositionalsErr(t *testing.T) {
+	errArgs1 := []string{"posint", "abc", "abc", "1.0"}
+	parser := NewParser("posint", "")
+	_ = parser.IntPositional(nil)
+	_ = parser.StringPositional(nil)
+	_ = parser.FloatPositional(nil)
+
+	if err := parser.Parse(errArgs1); err == nil {
+		t.Error("String argument accepted for integer")
+	} else if err.Error() != "[_positionalArg_posint_1] bad integer value [abc]" {
+		t.Error(err.Error())
+	}
+}
+
+// Just test we don't panic on add
+// Actual I/O during unit tests already covered by TestFileSimple1
+func TestFilePositional(t *testing.T) {
+	parser := NewParser("pos", "")
+	t1 := parser.FilePositional(os.O_RDWR, 0666, nil)
+	t2 := parser.FilePositional(os.O_RDWR, 0666, &Options{Help: "beep!"})
+
+	if t1 == nil {
+		t.Error("File pos was nil")
+	} else if t2 == nil {
+		t.Error("File pos was nil")
+	}
+}
+
+func TestPos1(t *testing.T) {
+	testArgs1 := []string{"pos", "subcommand1", "-i", "2", "abc"}
+	parser := NewParser("pos", "")
+
+	strval := parser.StringPositional(nil)
+	com1 := parser.NewCommand("subcommand1", "beep")
+	intval := com1.Int("i", "integer", nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "abc" {
+		t.Error("Strval did not match expected")
+	} else if *intval != 2 {
+		t.Error("intval did not match expected")
+	}
+}
+
+func TestPos2(t *testing.T) {
+	testArgs1 := []string{"pos", "subcommand1", "a123"}
+	parser := NewParser("pos", "")
+
+	strval := parser.StringPositional(nil)
+	com1 := parser.NewCommand("subcommand1", "beep")
+	intval := com1.Int("i", "integer", nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "a123" {
+		t.Error("Strval did not match expected")
+	} else if *intval != 0 {
+		t.Error("intval did not match expected")
+	}
+}
+
+func TestPos3(t *testing.T) {
+	testArgs1 := []string{"pos", "subcommand1", "xyz", "--integer", "3"}
+	parser := NewParser("pos", "")
+
+	strval := parser.StringPositional(nil)
+	com1 := parser.NewCommand("subcommand1", "beep")
+	intval := com1.Int("i", "integer", nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "xyz" {
+		t.Error("Strval did not match expected")
+	} else if *intval != 3 {
+		t.Error("intval did not match expected")
+	}
+}
+
+func TestPos4(t *testing.T) {
+	testArgs1 := []string{"pos", "abc"}
+	parser := NewParser("pos", "")
+
+	strval := parser.StringPositional(nil)
+	com1 := parser.NewCommand("subcommand1", "beep")
+	intval := com1.Int("i", "integer", nil)
+
+	if err := parser.Parse(testArgs1); err == nil {
+		t.Error("Error expected")
+	} else if err.Error() != "[sub]Command required" {
+		t.Error(err.Error())
+	} else if *strval != "" {
+		t.Error("Strval did not match expected")
+	} else if *intval != 0 {
+		t.Error("intval did not match expected")
+	}
+}
+
+// Test is covering internal logical error
+func TestPos5(t *testing.T) {
+	errStr := "unable to add Flag: argument type cannot be positional"
+	parser := NewParser("pos", "")
+	var boolval *bool
+	// Catch the panic
+	defer func() {
+		err := recover()
+		if err.(error).Error() != errStr {
+			t.Error(err.(error).Error())
+		} else if boolval != nil {
+			t.Error("Boolval was set")
+		}
+	}()
+	boolval = parser.Flag("", "booly", &Options{positional: true})
+}
+
+func TestPos6(t *testing.T) {
+	testArgs1 := []string{"pos", "subcommand1", "-i=2", "abc"}
+	parser := NewParser("pos", "")
+
+	strval := parser.StringPositional(nil)
+	com1 := parser.NewCommand("subcommand1", "beep")
+	intval := com1.Int("i", "integer", nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "abc" {
+		t.Error("Strval did not match expected")
+	} else if *intval != 2 {
+		t.Error("intval did not match expected")
+	}
+}
+
+func TestPos7(t *testing.T) {
+	testArgs1 := []string{"pos", "beep"}
+	parser := NewParser("pos", "")
+
+	strval := parser.SelectorPositional([]string{"beep"}, &Options{Help: "wow"})
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	} else if *strval != "beep" {
+		t.Error("Strval did not match expected")
+	}
+}
+
+func TestPos8(t *testing.T) {
+	testArgs1 := []string{"pos", "cmd1", "cmd2", "progPos", "cmd1pos1", "-s", "some string", "cmd1pos2", "cmd2pos1"}
+	parser := NewParser("pos", "")
+
+	cmd1 := parser.NewCommand("cmd1", "")
+	cmd2 := cmd1.NewCommand("cmd2", "")
+
+	// The precedence of commands is playing a role here.
+	// We should be parsing in root->leaf, left->right order
+	cmd2pos1 := cmd2.StringPositional(nil)
+	progPos := parser.StringPositional(nil)
+	cmd1pos1 := cmd1.StringPositional(nil)
+	strval := cmd1.String("s", "str", nil)
+	cmd1pos2 := cmd1.StringPositional(nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	}
+
+	if !cmd1.Happened() {
+		t.Errorf("cmd1 not happened")
+	}
+	if !cmd2.Happened() {
+		t.Errorf("cmd2 not happened")
+	}
+	if *strval != "some string" {
+		t.Errorf(`*strval expected "some string", but got "%s"`, *strval)
+	}
+	if *progPos != "progPos" {
+		t.Errorf(`*progPos expected "progPos", but got "%s"`, *progPos)
+	}
+	if *cmd1pos1 != "cmd1pos1" {
+		t.Errorf(`*cmd1pos1 expected "cmd1pos1", but got "%s"`, *cmd1pos1)
+	}
+	if *cmd1pos2 != "cmd1pos2" {
+		t.Errorf(`*cmd1pos2 expected "cmd1pos1", but got "%s"`, *cmd1pos2)
+	}
+	if *cmd2pos1 != "cmd2pos1" {
+		t.Errorf(`*cmd2pos1 expected "cmd2pos1", but got "%s"`, *cmd2pos1)
+	}
+}
+
+func TestPos9(t *testing.T) {
+	testArgs1 := []string{"pos", "cmd1", "cmd2", "progPos", "cmd1pos1", "-s", "some string", "cmd1pos2", "cmd2pos1"}
+	parser := NewParser("pos", "")
+
+	cmd1 := parser.NewCommand("cmd1", "")
+	cmd2 := cmd1.NewCommand("cmd2", "")
+
+	// The precedence of commands controls which values parsed to where
+	// We should be parsing in root->leaf, left->right order
+	cmd2pos1 := cmd2.StringPositional(nil)
+	progPos := parser.StringPositional(nil)
+	cmd1pos1 := cmd1.StringPositional(nil)
+	cmd1pos2 := cmd1.StringPositional(nil)
+
+	strval := cmd1.String("s", "str", nil)
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	}
+
+	if !cmd1.Happened() {
+		t.Errorf("cmd1 not happened")
+	}
+	if !cmd2.Happened() {
+		t.Errorf("cmd2 not happened")
+	}
+	if *strval != "some string" {
+		t.Errorf(`*strval expected "some string", but got "%s"`, *strval)
+	}
+	if *progPos != "progPos" {
+		t.Errorf(`*progPos expected "progPos", but got "%s"`, *progPos)
+	}
+	if *cmd1pos1 != "cmd1pos1" {
+		t.Errorf(`*cmd1pos1 expected "cmd1pos1", but got "%s"`, *cmd1pos1)
+	}
+	if *cmd1pos2 != "cmd1pos2" {
+		t.Errorf(`*cmd1pos2 expected "cmd1pos1", but got "%s"`, *cmd1pos2)
+	}
+	if *cmd2pos1 != "cmd2pos1" {
+		t.Errorf(`*cmd2pos1 expected "cmd2pos1", but got "%s"`, *cmd2pos1)
+	}
+}
+
+func TestSubcommandParsed(t *testing.T) {
+	errArgs1 := []string{"pos", "subcommand1"}
+	parser := NewParser("pos", "")
+
+	strval := parser.StringPositional(nil)
+	com1 := parser.NewCommand("subcommand1", "beep")
+	intval := com1.Int("i", "integer", nil)
+
+	if err := parser.Parse(errArgs1); err != nil {
+		t.Error(err.Error())
+	} else if !com1.Happened() {
+		t.Error("Subcommand should have happened")
+	} else if *strval != "" {
+		t.Error("strval incorrectly defaulted:" + *strval)
+	} else if *intval != 0 {
+		t.Error("intval did not match expected")
+	}
+}
+
+func TestSubcommandMultiarg(t *testing.T) {
+	errArgs1 := []string{"ma0", "ma1", "ma2", "strval1", "2.0", "5", "1.0"}
+	parser := NewParser("ma0", "")
+
+	strval := parser.StringPositional(nil)
+	floatval1 := parser.FloatPositional(nil)
+	com1 := parser.NewCommand("ma1", "beep")
+	intval := com1.IntPositional(nil)
+	com2 := com1.NewCommand("ma2", "beep")
+	floatval2 := com2.FloatPositional(nil)
+
+	if err := parser.Parse(errArgs1); err != nil {
+		t.Error(err.Error())
+	} else if !com1.Happened() {
+		t.Error("ma1 should have happened")
+	} else if !com2.Happened() {
+		t.Error("ma2 should have happened")
+	} else if *strval != "strval1" {
+		t.Error("strval did not match expected")
+	} else if *floatval1 != 2.0 {
+		t.Error("strval did not match expected")
+	} else if *intval != 5 {
+		t.Errorf("intval did not match expected: %v", *intval)
+	} else if *floatval2 != 1.0 {
+		t.Error("floatval did not match expected")
+	}
+}
+
+func TestCommandSubcommandPositionals(t *testing.T) {
+	testArgs1 := []string{"pos", "subcommand2", "efg"}
+	testArgs2 := []string{"pos", "subcommand1"}
+	testArgs3 := []string{"pos", "subcommand2", "abc", "-i", "1"}
+	testArgs4 := []string{"pos", "subcommand2", "abc", "--integer", "1"}
+	testArgs5 := []string{"pos", "subcommand2", "abc", "-i=1"}
+	testArgs6 := []string{"pos", "subcommand2", "abc", "--integer=1"}
+	// flags before positional must use `=` for values
+	testArgs7 := []string{"pos", "subcommand2", "-i=1", "abc"}
+	testArgs8 := []string{"pos", "subcommand2", "--integer=1", "abc"}
+	testArgs9 := []string{"pos", "subcommand3", "second"}
+	testArgs10 := []string{"pos", "subcommand2", "-i", "1", "abc"}
+	testArgs11 := []string{"pos", "subcommand2", "-i", "1"}
+	// Error cases
+	errArgs1 := []string{"pos", "subcommand3", "abc"}
+
+	newParser := func() *Parser {
+		parser := NewParser("pos", "")
+		_ = parser.NewCommand("subcommand1", "")
+		com2 := parser.NewCommand("subcommand2", "")
+		com2.StringPositional(nil)
+		com2.Int("i", "integer", nil)
+		com2.Flag("b", "bool", nil)
+		com3 := parser.NewCommand("subcommand3", "")
+		com3.SelectorPositional([]string{"first", "second"}, nil)
+		return parser
+	}
+
+	if err := newParser().Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs2); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs3); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs4); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs5); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs6); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs7); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs8); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs9); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs10); err != nil {
+		t.Error(err.Error())
+	}
+	if err := newParser().Parse(testArgs11); err != nil {
+		t.Error(err.Error())
+	}
+
+	if err := newParser().Parse(errArgs1); err == nil {
+		t.Error("Expected error")
+	}
+}
+
+func TestPositionalsLessArgumentsThanPositionals(t *testing.T) {
+	testArgs1 := []string{"pos", "cmd1", "progPos", "cmd1pos1"}
+	parser := NewParser("pos", "")
+
+	cmd1 := parser.NewCommand("cmd1", "")
+
+	// The precedence of commands is playing a role here.
+	// We should be parsing in root->leaf, left->right order
+	progPos := parser.StringPositional(nil)
+	cmd1pos1 := cmd1.StringPositional(nil)
+	cmd1pos2 := cmd1.StringPositional(nil)
+	strval := cmd1.String("s", "str", nil)
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	}
+
+	if !cmd1.Happened() {
+		t.Errorf("cmd1 not happened")
+	}
+	if *strval != "" {
+		t.Errorf(`*strval expected "", but got "%s"`, *strval)
+	}
+	if *progPos != "progPos" {
+		t.Errorf(`*progPos expected "progPos", but got "%s"`, *progPos)
+	}
+	if *cmd1pos1 != "cmd1pos1" {
+		t.Errorf(`*cmd1pos1 expected "cmd1pos1", but got "%s"`, *cmd1pos1)
+	}
+	if *cmd1pos2 != "" {
+		t.Errorf(`*cmd1pos2 expected "", but got "%s"`, *cmd1pos2)
+	}
+}
+
+func TestPositionalDefaults(t *testing.T) {
+	testArgs1 := []string{"pos"}
+	parser := NewParser("pos", "")
+
+	pos1 := parser.StringPositional(&Options{Default: "pos1"})
+	pos2 := parser.IntPositional(&Options{Default: 2})
+	pos3 := parser.FloatPositional(&Options{Default: 3.3})
+	pos4 := parser.SelectorPositional([]string{"notallowed", "pos4"}, &Options{Default: "pos4"})
+
+	if err := parser.Parse(testArgs1); err != nil {
+		t.Error(err.Error())
+	}
+
+	if *pos1 != "pos1" {
+		t.Errorf(`*pos1 expected "pos1", but got "%s"`, *pos1)
+	}
+	if *pos2 != 2 {
+		t.Errorf(`*pos2 expected "2", but got "%d"`, *pos2)
+	}
+	if *pos3 != 3.3 {
+		t.Errorf(`*pos3 expected "3.3", but got "%f"`, *pos3)
+	}
+	if *pos4 != "pos4" {
+		t.Errorf(`*pos4 expected "pos4", but got "%s"`, *pos4)
+	}
+}
